#include <stdio.h>
#include <stdlib.h>
#include "avgchecknull.h"


void avg_checknull(void *ptr, char *x, char *y)
{
  if (ptr == NULL) {
    fprintf(stderr,"Pointer %s in function %s is NULL\n",y,x);
    fflush(stderr);
  }
}
