/****Copyright (c) 1997-2009 All rights reserved
 ****A. V. Gerbessiotis 
 ****Permission to use, copy, modify, and distribute this software,
 ****and to incorporate it, in whole or in part, into 
 ****other software,is hereby granted without fee, provided that
 ****  (1) the above copyright notice and this permission 
 ****      notice appear in all copies of the source code, and 
 ****      the above copyright notice appear in clearly visible 
 ****      form on all supporting documentation and distribution 
 ****      media, and
 ****     (2) any redistribution of the software, in original
 ****         or modified form, be without fee and subject to
 ****         these same conditions.
 ****   No guarantee, whatsoever, is offered that the code works as
 ****   advertised or is absent of any, even damaging,
 ****   side-effects. Use this code at your own personal risk.
 ****/
#include "ssrt.h"

void g36s(void *akeys,int n,int size,int (*compare)(const void*,const void*),int l,int r)
{
 register int i,j,M;
 char	*tmp, *keys,*SRC,*DES;

 if ((n<=1) ||(size<=0)) return;
 keys = (char *) akeys;
 tmp = (char *) malloc(size*sizeof(char));
#ifdef SANITY_CHECK
 check_if_null(temp,"g36s","tmp");
#endif
 i=l;j=2*i+1;
 MEMCPY(tmp,&keys[i*size],size);
 while(j<=r){
   if(j<r){ 
     if(compare(&keys[j*size],&keys[(j+1)*size])<0){
       j++;
     }
   }
   if (compare(tmp,&keys[(j)*size]) >= 0)
	break;

   MEMCPY(&keys[i*size],&keys[j*size],size);
   i=j;j=2*i+1;
 }
 MEMCPY(&keys[i*size],tmp,size);
 free((void*)tmp);
}
	

void g36heapx(void *akeys,int n,int size,int (*compare) (const void*,const void*))
{
 register int l,r,M;
 char	*tmp,*keys,*SRC,*DES;

 keys=(char *)akeys;
 l=n/2+1;r=n-1;
 tmp=(char *)malloc(size*sizeof(char));
 while(l>0){
   l--;
   g36s(keys,n,size,compare,l,r);
 }
 while (r > 0) {
   MEMCPY(tmp,&keys[0],size);
   MEMCPY(keys,&keys[r*size],size);
   MEMCPY(&keys[r*size],tmp,size);
   r--;
   g36s(keys,n,size,compare,l,r);
 }
 free((void*)tmp);
}
