#include <sys/time.h>

#define AVG_TINIT   AVG_PERRUN = AVG_ELAPSED =(double)0;
#define AVG_TSTART  AVG_PERRUN = (double) 0.0 ; avg_tstart();
#define AVG_TEND    avg_tend(); AVG_PERRUN = (avg_treturn() ); AVG_ELAPSED +=(AVG_PERRUN);
#define AVG_PERRUN  per_run
#define AVG_ELAPSED elapsed

void   avg_tstart(void);
void   avg_tend(void);
double avg_treturn(void);

