/* July 14, 2008 v1 */

#define Malloc(x,y)   avg_malloc((x),(y))
#define Free(x,y,z)   avg_free  ((x),(y),(z))

void *avg_malloc (size_t , char *);
void  avg_free   (void *, int , char *);
void  avg_checknull   (void *, char *,char*);

#undef  TRACE_MALLOC
#define TRACE_MALLOC
