/****Copyright (c) 1997-2005 All rights reserved
 ****A. V. Gerbessiotis 
 ****Permission to use, copy, modify, and distribute this software,
 ****and to incorporate it, in whole or in part, into 
 ****other software,is hereby granted without fee, provided that
 ****  (1) the above copyright notice and this permission 
 ****      notice appear in all copies of the source code, and 
 ****      the above copyright notice appear in clearly visible 
 ****      form on all supporting documentation and distribution 
 ****      media, and
 ****     (2) any redistribution of the software, in original
 ****         or modified form, be without fee and subject to
 ****         these same conditions.
 ****   No guarantee, whatsoever, is offered that the code works as
 ****   advertised or is absent of any, even damaging,
 ****   side-effects. Use this code at your own personal risk.
 ****/
/* A bubble sort */
#include "ssrt.h"

void g25bub(void *akeys, int n, int size, int (*compare) (const void*,const void*))
{
 register int i,j,k,M;
 register char	*tmp, *keys,*DES,*SRC;

 if (n<=1) return;
 keys=(char *)akeys;
 tmp = (char *) malloc(size*sizeof(char));
#ifdef SANITY_CHECK
  check_if_null(tmp,"g25bub","tmp");
#endif
 for(i=1;i<n;i++) {
   k=i;
   MEMCPY(tmp,&keys[i*size],size);
   for(j=n-1;j>=i;j--) {
     if (compare(&keys[(j-1)*size],&keys[j*size]) > 0) {
       MEMCPY(tmp,&keys[(j-1)*size],size);
       MEMCPY(&keys[(j-1)*size],&keys[j*size],size);
       MEMCPY(&keys[j*size],tmp,size);
     }
   }
 }
 free((void*)tmp);
}
