/****Copyright (c) 1997-2005 All rights reserved
 ****A. V. Gerbessiotis and C. J. Siniolakis
 ****Permission to use, copy, modify, and distribute this software,
 ****and to incorporate it, in whole or in part, into 
 ****other software,is hereby granted without fee, provided that
 ****  (1) the above copyright notice and this permission 
 ****      notice appear in all copies of the source code, and 
 ****      the above copyright notice appear in clearly visible 
 ****      form on all supporting documentation and distribution 
 ****      media, and
 ****     (2) any redistribution of the software, in original
 ****         or modified form, be without fee and subject to
 ****         these same conditions.
 ****   No guarantee, whatsoever, is offered that the code works as
 ****   advertised or is absent of any, even damaging,
 ****   side-effects. Use this code at your own personal risk.
 ****/
#include "smrg.h"
#include <stdio.h>
#include <stdlib.h>
#include <memory.h>

int main(void)
{
 int a[10],i;
 int *a1, *a2;
 for(i=0;i<10;i++) a[i]=i+1;

 a1= &a[0];
 a2= &a[4];
 printf("%u %u %d %d %d\n",(unsigned int)a1,(unsigned int)a2,a2-a1,*a1,*a2);
 return(1);
}
