/****Copyright (c) 1997-2005 All rights reserved
 ****A. V. Gerbessiotis 
 ****Permission to use, copy, modify, and distribute this software,
 ****and to incorporate it, in whole or in part, into 
 ****other software,is hereby granted without fee, provided that
 ****  (1) the above copyright notice and this permission 
 ****      notice appear in all copies of the source code, and 
 ****      the above copyright notice appear in clearly visible 
 ****      form on all supporting documentation and distribution 
 ****      media, and
 ****     (2) any redistribution of the software, in original
 ****         or modified form, be without fee and subject to
 ****         these same conditions.
 ****   No guarantee, whatsoever, is offered that the code works as
 ****   advertised or is absent of any, even damaging,
 ****   side-effects. Use this code at your own personal risk.
 ****/
#include "ssrt.h"

void g11ins(void *akeys, int n, int size, int (*compare) (const void*,const void*))
{
 register int i,j;
 register char *keys,*tmp, *p;

 if (n <=1) return;
 keys=(char *)akeys;
 tmp = (char *) malloc(size*sizeof(char));
#ifdef SANITY_CHECK
  check_if_null(tmp,"g11ins","tmp");
#endif
 for(j=1;j<n;j++) {
   MEMCPY(tmp,(keys+j*size),size);
   i=j-1; p= keys+i*size;
   while ( (i>=0) && ( compare(p,tmp)>0) ) {
     MEMCPY((p+size),(p),size);
     i--;p-=size;
   }
   MEMCPY((p+size),tmp,size);
 }
 free((void*)tmp);
}
